<?php
/**
 * Process file upload.
 *
 * @param string $name The file input name.
 * @return array
 */
function process_image_upload( $name = 'image-custom' ) {
	$result = array(
		'success' => false,
		'error'   => '',
		'image'   => '',
	);

	if ( ! empty( $_FILES[ $name ] ) && ! empty( $_FILES[ $name ]['name'] )
		&& 'none' !== $_FILES[ $name ]['name'] && empty( $_FILES[ $name ]['error'] ) ) {
		/*
		 Here you should add your server-size validation.
		 ...
		 */

		// This is just an example, the file will be uploaded in the same folder as the script.
		$dirn = dirname( __FILE__ ) . '/';
		$file = 'image-custom.' . strtolower( pathinfo( $_FILES[ $name ]['name'], PATHINFO_EXTENSION ) );

		if ( ! @move_uploaded_file( $_FILES[ $name ]['tmp_name'], $dirn . $file ) ) {
			$result['error'] = 'ERROR! The file <strong>' . addslashes( $_FILES[ $name ]['name'] ) . '</strong> could not be saved.';

			// No need to continue, return here the error.
			return $result;
		} else {
			$result['success'] = true;
			$result['image'] = $file;
			$result['output_image'] = './' . $result['image'] . '?v= ' . time();
		}
	}

	// Return the array.
	return $result;
}

// Listen for the custom uploader data and attempt to process the upload.
$maybe_process = filter_input( INPUT_POST, 'process-custom', FILTER_DEFAULT );
if ( 'on' === $maybe_process ) {
	$result = process_image_upload( 'image-custom' );

	// Output the JSON result.
	echo json_encode( $result );
	die();
}

?>
<!DOCTYPE html><html lang="ro">
<head>
	<meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1" />
	<meta charset="UTF-8">
	<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
	<meta http-equiv="content-language" content="en-EN" />
	<title>Custom AJAX Upload</title>
	<script
		src="https://code.jquery.com/jquery-3.4.1.min.js"
		integrity="sha256-CSXorXvZcTkaix6Yvo6HppcZGetbYMGWSFlBw8HfCJo="
		crossorigin="anonymous"></script>

	<style>
		html, body {
			font-family: 'Trebuchet MS';
			font-size: 16px;
			line-height: 18px;
		}

		* {
			box-sizing: border-box;
		}

		#image-custom {
			height: 0.1px;
			position: absolute;
			opacity: 0;
			overflow: hidden;
			width: 0.1px;
			z-index: -1;
		}

		#image-custom-label {
			background-color: #FFF;
			border: 1px solid #AAA;
			border-radius: 5px;
			cursor: pointer;
			display: block;
			width: 60%;
		}

		#image-custom-label.valid {
			background-color: #b0e37e;
		}
		#image-custom-label.invalid {
			background-color: #fff39b;
		}

		#upload-button {
			background-color: #DDD;
			border: 1px solid #AAA;
			border-radius: 5px;
			cursor: pointer;
			display: block;
			margin-left: 1%;
			width: 39%;
		}

		input, #image-custom-label, #upload-button {
			line-height: 32px;
			font-size: 16px;
			height: 32px;
			padding: 0 24px;
			float: left;
		}

		.processing {
			color: #555;
			opacity: 0.5 !important;
		}

		#input-wrap {
			background-color: #CCC;
			border: 5px dashed #CCC;
			min-height: 100px;
			padding: 32px;
			position: relative;
		}

		#input-wrap:after, #upload-button:after {
			clear: both;
			content: ' ',
			display: table;
			position: relative;
			width: 100%;
		}

		#input-wrap.file-dragged {
			border: 5px dashed #AAA;
		}

		#input-wrap.file-dragged:before {
			background-color: rgba(255,255,255,0.6);
			content: '\01F9F2';
			height: 100%;
			line-height: 50vh;
			left:0;
			position: absolute;
			width: 100%;
			top:0;
			text-align: center;
			z-index: 0;
		}

		#upload-results {
			background-color: #EEE;
			text-align: center;
		}

		#upload-results img {
			margin: 32px;
			max-width: 320px;
		}
	</style>
</head>
<body>

	<!--

	Description of the form main configuration:
	- action - the server-side script that handles the uploaded file
	- data-size-error - the attribute that records the file size errors
	- data-type-error - the attribute that records the file type errors
	- data-source-id - the element that contains the form elements that act like the pretty input
	- data-target-id - the element that will display the server-side response after the AJAX request is processed
	- data-button-id - the form submit button
	- data-label-id - the pretty label that replaces visually the file input
	- data-auto - if set to true, the form will auto upload the valid files, without clicking the upload button

	Description of the file input configuration:
	- data-extension- the list of file extensions, separated with comma (no spaces)
	- data-size - the maximum size for the file

	-->
	<form name="frm-custom"
		id="frm-custom"
		method="post"
		action="custom-ajax-upload.php"
		enctype="multipart/form-data"
		data-size-error=""
		data-type-error=""
		data-source-id="#input-wrap"
		data-target-id="#upload-results"
		data-button-id="#upload-button"
		data-label-id="#image-custom-label"
		data-auto="true"
		>
		<input type="hidden"
			name="process-custom"
			id="process-custom"
			value="on">
		<div id="upload-info">
			<p><h1>Custom AJAX Upload</h1></p>
			<p>The upload demo accepts the following file types: <b>jpg, jpeg, png or gif</b>.</p>
			<p>Upload a file of maximum <b>4MB</b>.</p>
		</div>
		<div id="input-wrap">
			<input type="file"
				name="image-custom"
				id="image-custom"
				data-extension="jpg,jpeg,gif,png"
				data-size="4MB">
			<label for="image-custom" id="image-custom-label">Select or drag a file</label>

			<input type="submit"
				name="upload-button"
				id="upload-button"
				value="Upload">
		</div>
		<div id="upload-results"></div>
	</form>

	<script>
		(function($) {
			window.CustomUploadExample = {
				form: '#frm-custom',
				input: '#image-custom',
				label: '#image-custom-label',
				source: '#input-wrap',
				target: '#upload-results',
				button: '#upload-button',
				auto: false,
				init: function(idForm,idInput) {
					// Initiate the properties.
					this.form = $(idForm);
					this.input = $(idInput);
					this.label = $(this.form.data('label-id'));
					this.source = $(this.form.data('source-id'));
					this.target = $(this.form.data('target-id'));
					this.button = $(this.form.data('button-id'));
					this.auto = (true === this.form.data('auto')) ? true: false;
					this.dragged = false;
					this.default = 'Select or drag a file';

					// Hookup the custom functionality.
					this.hookupChange();
					this.hookupDrop();
					this.hookupSubmit();
				},
				hookupChange: function() {
					// Compute one time the restriction.
					this.input.data('filesize', this.sizeConvertor(this.input.data('size')));

					// Make the necessary actions on file change.
					var self = this;
					this.input.on('change', function() {
						self.dragged = false;
						self.assessFile();
						self.displayHints();
						self.assessAuto();
					});
				},
				hookupDrop: function() {
					var self = this;
					this.form.on('dragover dragenter', function(e) {
						self.source.addClass('file-dragged');
						e.preventDefault();
					});

					this.form.on('drop', function(e) {
						e.preventDefault();
						if (e.originalEvent.dataTransfer.files[0]) {
							self.dragged = e.originalEvent.dataTransfer.files[0];
							self.label.html(self.dragged.name);
							self.assessFile();
							self.displayHints();
							self.assessAuto();
						}
						self.source.removeClass('file-dragged');
					});
				},
				hookupSubmit: function() {
					// Make the necessary actions on form submit.
					var self = this;
					this.form.on('submit', function(e) {
						e.preventDefault();
						if (self.formValid()) {
							self.executeUpload();
						} else {
							self.displayHints();
						}
					});
				},
				assessAuto: function() {
					if (this.formValid() && this.auto) {
						this.executeUpload();
					}
				},
				assessFile: function() {
					var file = this.input.prop('files')[0];
					if (this.dragged) {
						// There is a dragged file, use this one.
						file = this.dragged;
					}
					if (file) {
						this.assessFileSize(file);
						this.assessFileType(file);
						if (this.formValid()) {
							this.label.removeClass('invalid').addClass('valid');
						} else {
							this.label.removeClass('valid').addClass('invalid');
						}
					}
				},
				assessFileSize: function(file) {
					if (file) {
						this.label.html(file.name);
						if (file.size > this.input.data('filesize')) {
							this.form.data('size-error', 'Maximum allowed size is ' + this.input.data('size') + '.');
						} else {
							this.form.data('size-error', '');
						}
					}
				},
				assessFileType: function(file) {
					if (file) {
						var extensions = this.input.data('extension');
						var allow = extensions.split(',');
						if (! allow.includes('*')) {
							var value = file.name.toLowerCase();
							var ext = value.split('.').pop();
							if (! allow.includes(ext)) {
								this.form.data('type-error', 'File types allowed: ' + extensions.replace(/,/g, ', ') + '.');
							} else {
								this.form.data('type-error', '');
								this.label.html(file.name);
							}
						}
					}
				},
				formValid: function() {
					var error = this.form.data('size-error') + this.form.data('type-error');
					if (error) {
						return false;
					} else {
						return true;
					}
				},
				displayHints: function() {
					$('#upload-hints').remove();
					if (! this.formValid(this.form)) {
						this.source.append('<div id="upload-hints" class="type-error">' + this.form.data('size-error') + ' ' + this.form.data('type-error') + '</div>');
						this.button.attr('disabled', true);
					} else {
						this.button.removeAttr('disabled');
					}
				},
				sizeConvertor: function(size) {
					if (size.endsWith('MB')) {
						// Put this first, is the most common.
						return parseFloat(size) * 1024 * 1024;
					} else if (size.endsWith('KB')) {
						return parseFloat(size) * 1024;
					} else if (size.endsWith('Bytes') || size.endsWith('B')) {
						return size;
					} else if (size.endsWith('GB')) {
						return parseFloat(size) * 1024 * 1024 * 1024;
					} else if (size.endsWith('TB')) {
						return parseFloat(size) * 1024 * 1024 * 1024 * 1024;
					}
					return parseFloat(size);
				},
				initAjax: function() {
					this.source.addClass('processing');
					this.source.append('<div id="upload-processing">The request is processing...</div>');
					this.target.find('.type-error').remove();
				},
				finishAjax: function() {
					this.source.removeClass('processing');
					this.source.find('#upload-processing').remove();
					this.label.removeClass('invalid valid').html(this.default);
					this.input.val('');
				},
				executeUpload: function() {
					var self = this;
					this.initAjax();
					var args = new FormData(this.form[0]);
					if (this.dragged) {
						args.delete('image-custom');
						args.append('image-custom', this.dragged);
					}
					var url  = this.form.attr('action');
					jQuery.ajax({
						type: 'POST',
						url: url,
						data: args,
						cache: false,
						contentType: false,
						processData: false,
						dataType: 'json',
					}).done( function( response ) {
						if(true == response.success) {
							self.target.prepend('<div id="upload-result"><a href="' + response.output_image + '" target="_blank"><img src="' + response.output_image + '"></a></div>');
						} else {
							self.target.prepend('<div class="type-error">' + response.error + '</div>');
						}
						self.finishAjax();
					}).fail( function( response ) {
						self.target.prepend('<div class="type-error">There was an error processing the file.</div>');
						self.finishAjax();
					});
				}
			}

			// This function will initialize the custom upload (page has loaded).
			$(document).ready(function(){
				CustomUploadExample.init('#frm-custom', '#image-custom');
			});
		})(jQuery);

	</script>

</body>
</html>
